/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef EMERGENCYNUMBERS_H
#define EMERGENCYNUMBERS_H

#include <QtCore/QObject>

#include "libcellular-qt_global.h"

namespace Cellular
{

class EmergencyNumbersPrivate;

//! Emergency numbers.
/*!
 * This class provides access to the current set of usable emergency
 * phone numbers.
 */
class CELLULAR_QT_EXPORT EmergencyNumbers : public QObject
{

Q_OBJECT

public:
    //! Possible service categories for a number
    enum ServiceCategory {
	NoCategory = 0,        //!< No category attached to this number.
	Police = 0x01,         //!< Emergency number to reach the police.
	Ambulance = 0x02,      //!< Emergency number to reach an ambulance.
	FireBrigade = 0x04,    //!< Emergency number to reach the fire brigade.
	MarineGuard = 0x08,    //!< Emergency number to reach the marine guard.
	MountainRescue = 0x10  //!< Emergency number to reach the mountain rescue.
    };
    Q_DECLARE_FLAGS(ServiceCategories, ServiceCategory)
    
    //! Possible origins for a number
    enum NumberOrigin {
	HardCoded = 0x00,      //!< Emergency number is hard-coded.
	SIM = 0x1,             //!< Emergency number was read from the SIM card.
	PermanentMemory = 0x2  //!< Emergency number was read from permanent memory.
    };
    Q_DECLARE_FLAGS(NumberOrigins, NumberOrigin)

    //! Full information about an emergency number
    struct EmergencyNumberInfo {
        QString number; //!< The emergency number itself.
        ServiceCategories serviceCategories; //!< Service category for this emergency number.
        NumberOrigins numberOrigins; //!< Origins for this emergency number.
    };

    //! A list of available emergency numbers    
    typedef QList<EmergencyNumberInfo> EmergencyNumbersList;


public:
    EmergencyNumbers(QObject *parent=0);
    ~EmergencyNumbers();

    //! Get the current set of available, usable emergency numbers without additional information.
    QStringList numbers() const;

    //! Get the current set of available, usable emergency numbers with additional information.
    EmergencyNumbersList numbersWithInfo() const;

    //! Check validity of class instance.
    bool isValid() const;

private:
    EmergencyNumbersPrivate *d;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(EmergencyNumbers::ServiceCategories)
Q_DECLARE_OPERATORS_FOR_FLAGS(EmergencyNumbers::NumberOrigins)
}

#endif  /* !EMERGENCYNUMBERS_H */
