/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef NETWORKCELL_H
#define NETWORKCELL_H

#include <QtCore/QObject>
#include <QtCore/QFlags>

#include "libcellular-qt_global.h"

namespace Cellular
{

class NetworkCellPrivate;

//! Network cell.
/*!
 * This class provides information on the current network cell; its
 * type, available services, both an integer identity and possibly a
 * human-readable cell name; and Location Area Core (LAC).
 */
class CELLULAR_QT_EXPORT NetworkCell : public QObject
{

    Q_OBJECT
    Q_PROPERTY(Type type READ type)
    Q_PROPERTY(Services services READ services)
    Q_PROPERTY(int id READ id)
    Q_PROPERTY(int lac READ lac)
    Q_PROPERTY(QString name READ name)
    Q_PROPERTY(bool display READ nameDisplayEnabled WRITE setNameDisplayEnabled)
    Q_PROPERTY(bool valid READ isValid)
    Q_ENUMS(Type)
    Q_FLAGS(Services)

public:
    NetworkCell(QObject *parent=0);
    ~NetworkCell();

    //! Type of the current network cell.
    /*!
     * This enum describes the currently serving network cell
     * types.
     */
    enum Type 
    { 
        UnknownType = -1, //!< Current cell type is unknown.
        GSMCell, //!< Current cell is a GSM cell.
        WCDMACell //!< Current cell is a WCDMA cell.
    };

    //! Supported services in the current cell.
    /*!
     * This enum describes the possible flags present in the
     * NetworkCell::Services flags.
     */
    enum Service
    {
	CSService = 1,         /*!< Support for Circuit Switched (CS)
				* services, i.e., plain old telephony
				* services such as short messages and
				* voice calls. */
	GPRSService = 1 << 1,  /*!< Supports General Packet Radio
				* Service (GPRS) services. */
	GPRSSupport = 1 << 2,  /*!< Supports General Packet Radio
				* Service (GPRS) data connections. */
	EGPRSSupport = 1 << 3, /*!< Support for enhanced data rates in
				* GPRS. */
	DTMSupport = 1 << 4,   /*!< Support for Dual Transfer Mode
				* (DTM). */
	HSDPASupport = 1 << 5, /*!< Support for High Speed Downlink
				* Packet Access (HSDPA). */
	HSUPASupport = 1 << 6  /*!< Support for High Speed Uplink
				* Packet Access (HSUPA). */
    };
    Q_DECLARE_FLAGS(Services, Service)

    //! Get the current cell type.
    /*!
     * This method returns the type of the currently serving network
     * cell. A cell type of NetworkCell::UnknownType is an indication
     * that the device is currently not in service.
     *
     * \return Type of the currently serving network cell as
     * NetworkCell::Type.
     */
    Type type() const;

    //! Get the currently supported services in cell.
    /*!
     * This method returns properties of the currently serving network
     * cell. Namely, it describes which services or features the
     * current cell provides.
     *
     * \return NetworkCell::Services flags.
     */
    Services services() const;

    //! Get the current cell ID.
    /*!
     * This method returns the currently serving network cell ID or
     * the current UCID in case the serving cell is of type
     * NetworkCell::WCDMACell. For NetworkCell::GSMCell, a valid range
     * for the value is 0...65535; for NetworkCell::WCDMACell, the
     * valid range is 0...268435455. If the cell is currently
     * unavailable, -1 is returned.
     *
     * \return Identity of the currently serving network cell, or -1
     * if it is unavailable.
     */
    int id() const;

    //! Get the current cell Location Area Code (LAC).
    /*!
     * This method returns the currently serving network Location Area
     * Code (LAC). A valid range for the lac() is 0...65535; if the
     * cell is currently unavailable, -1 is returned.
     *
     * \return Location Area Code (LAC) of the currently serving
     * network cell, or -1 if it is unavailable.
     */
    int lac() const;

    //! Get the current cell name.
    /*!
     * This method returns the currently serving cell's human-readable
     * name. This name is received as a network broadcast using the
     * Cell Broadcast Service (CBS), and may not always be
     * available. If the name broadcast has not been received for the
     * current cell, an empty QString() is returned.
     *
     * \return A human readable name of the currently serving cell, or
     * an empty QString if such a name is not available.
     */
    QString name() const;

    //! Get the current name display preference.
    /*!
     * This method returns the current preference for displaying the
     * cell name. As such, it is merely a hint to the user interface
     * whether to display the cell name or not.
     *
     * \return True if cell name should be visible in the user
     * interface, False otherwise.
     */
    bool nameDisplayEnabled() const;

    //! Set the current name display preference.
    /*!
     * This method sets the current preference for displaying the cell
     * name.
     */
    //! @credential %Cellular resource token required
    void setNameDisplayEnabled(bool value);

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service was not responding and the properties are
     * not available. This is an indication something is seriously
     * wrong, and should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover. Perhaps deleting the class instance, and
     * trying again after a while might help.
     */
    bool isValid() const;

Q_SIGNALS:
    //! Cell has changed.
    /*!
     * This signal indicates that the currently serving cell has
     * changed.
     *
     * \param id The identity of the new serving cell.
     * \param lac The location area code of the new serving cell.
     * \param type The NetworkCell::Type of the new serving cell.
     */
    void cellChanged(int id, int lac, int type);

    //! Cell name has changed.
    /*!
     * This signal indicates that the human readable name of the cell
     * has changed.
     *
     * \param name String name of the current cell.
     */
    void cellNameChanged(const QString &name);

    //! Cell services changed.
    /*!
     * This signal indicates that the services that the currently
     * serving cell provides have changed.
     *
     * \param flags NetworkCell::Services flags describing the new service properties.
     */
    void servicesChanged(NetworkCell::Services flags);

    //! Cell name display preference changed.
    /*!
     * This signal indicates that the cell name display preference has
     * changed.
     */
    void cellNameDisplayEnabledChanged(bool value);

private:
    NetworkCellPrivate *d;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(NetworkCell::Services)

}

#endif  /* !NETWORKCELL_H */
