/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef NETWORKOPERATOR_H
#define NETWORKOPERATOR_H

#include <QtCore/QObject>
#include <QtCore/QString>

#include "libcellular-qt_global.h"

namespace Cellular
{

class NetworkOperatorPrivate;

//! Network operator.
/*!
 * This class provides information on the currently serving network
 * operator. It allows checking the Mobile Network Core (MNC) and
 * Mobile Country Code (MCC) values, as well as the name of the
 * operator.
 */
class CELLULAR_QT_EXPORT NetworkOperator : public QObject
{

    Q_OBJECT
    Q_PROPERTY(QString name READ name)
    Q_PROPERTY(QString mnc READ mnc)
    Q_PROPERTY(QString mcc READ mcc)
    Q_PROPERTY(bool valid READ isValid)

public:
    NetworkOperator(QObject *parent=0);
    ~NetworkOperator();

    //! Get current network operator name
    /*!
     * This operator name is according to standards, and may have been
     * composed from many pieces of information. For instance, if the
     * SIM card has the Service Provider Name (SPN) set, it will be
     * used as the primary name. If additional control bits are set,
     * it may or may not be accompanied with an operator name received
     * form the network, or a so-called hard-coded name that
     * originates from a list maintained by the GSM Association. The
     * display information also depends on current roaming status.
     *
     * The format is as follows:
     *
     *   provider-name ["(" other-name ")"]
     *
     * 'provider-name' is typically the SPN, except when
     * roaming. 'other-name' can be a network-given name, or a
     * hard-coded name. When roaming these roles are reversed.
     *
     * \return Display name for the current operator, or empty if not
     * available
     */
    QString name() const;

    //! Get current Mobile Network Code (MNC)
    /*!
     * This method returns the network code of the currently serving
     * network operator.
     *
     * There is a reason that the return value is a string and not an
     * unsigned. The network code can be 2 or 3 digits long, and
     * unfortunately leading zeros are not silent. I.e., an MNC of
     * '005' is not the same as '05'. Some regions use leading zeros
     * while others don't.
     *
     * \see http://en.wikipedia.org/wiki/Mobile_Network_Code
     *
     * \return A string containing the network code of the currently
     * serving network operator, or empty is not available.
     */
    QString mnc() const;

    //! Get current Mobile Country Code (MCC)
    /*!
     * This method returns the currently serving network operator's
     * country code. The MCC is always three digits long.
     *
     * \see http://en.wikipedia.org/wiki/Mobile_Country_Code
     *
     * \return A string containing the current country code, or empty
     * if not available.
     */
    QString mcc() const;
    
    //! Country initials lookup
    /*!
     * This function returns the 2-letter ISO country initials for a given
     * operator. 
     *
     * \param mcc Mobile Country Code
     * \param mnc Mobile Network Code
     * \return Returns country initials corresponding with the given operator.
     */
    QString country(QString mcc, QString mnc) const;

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service was not responding and the properties are
     * not available. This is an indication something is seriously
     * wrong, and should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover. Perhaps deleting the class instance, and
     * trying again after a while might help.
     */
    bool isValid() const;

Q_SIGNALS:

    //! Current network operator has changed
    /*!
     * This signal is emitted when the currently serving operator has
     * changed. When the device unregisters, or goes out of coverage,
     * the signal will contain empty strings.
     *
     * \param mnc The new network code
     * \param mcc The new country code
     */
    void operatorChanged(const QString &mnc, const QString &mcc);

    //! Current operator name has changed
    /*!
     * This signal is emitted when the currently serving operator's
     * name has changed. Typically this signal is sent right after the
     * operatorChanged() signal, but the name may also change without
     * the operator changing. For instance, a NITZ name can be
     * received from the network, or in some rare occasion, the SIM
     * card be updated with a new operator name. When the device
     * unregisters, or goes out of coverage, the signal will contain
     * an empty string.
     *
     * \param name New operator name
     */
    void nameChanged(const QString &name);

private:
    NetworkOperatorPrivate *d;
};

}

#endif  /* !NETWORKOPERATOR_H */
