/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef NETWORKREGISTRATION_H
#define NETWORKREGISTRATION_H

#include <QtCore/QObject>
#include <QtCore/QExplicitlySharedDataPointer>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QDataStream>

#include "libcellular-qt_global.h"

namespace Cellular
{

class AvailableOperatorData;
class NetworkRegistrationPrivate;

//! Available network operator.
/*!
 * This class models a network operator received via a scan for
 * available network operators. It includes information on the
 * preference and availability of the operator, as well as band
 * information, name, Mobile Network Code (MNC), Mobile Country Code
 * (MCC), and the availability of UMTS.
 *
 * The data in this class is explicitly shared, and reference counted,
 * using the QSharedData class. This data is referenced using a
 * QExplicitlySharedDataPointer as the d-pointer in this class.
 */
class CELLULAR_QT_EXPORT AvailableOperator
{

public:
    //! Operator availability.
    enum Availability
    {
	UnknownAvailability = -1, //!< Availability unknown
	Available, //!< Operator is available
	Current, //!< Operator is the current registered operator
	NotAvailable //!< Operator is not available
    };

    //! Operator band information.
    enum Band
    {
	UnknownBand = -1, //!< Band information not known
	AllSupported, //!< All supported bands available
	Gsm9001800, //!< GSM 900/1800 band
	Gsm8501900 //!< GSM 850/1900 band
    };

    //! Operator preference.
    enum Preference
    {
	UnknownPreference = -1, //!< Preference not known
	Home, //!< Home network operator
	Preferred, //!< Operator is a preferred roaming partner
	Forbidden //!< Operator is forbidden
    };

    AvailableOperator();
    AvailableOperator(const QString &mnc, const QString &mcc, const QString &name,
		      Band band, Availability availability, Preference preference,
		      bool umts);
    AvailableOperator(const AvailableOperator &other);
    ~AvailableOperator();

    //! Set operator MNC
    void setMnc(const QString &mnc);
    //! Set operator MCC
    void setMcc(const QString &mcc);
    //! Set operator Name
    void setName(const QString &name);
    //! Set operator availability
    void setAvailability(AvailableOperator::Availability availability);
    //! Set operator band information
    void setBand(AvailableOperator::Band band);
    //! Set oprerator preference
    void setPreference(AvailableOperator::Preference preference);
    //! Set operator UMTS availability status
    void setUmts(bool umts);

    //! Get operator MNC
    QString mnc() const;
    //! Get operator MCC
    QString mcc() const;
    //! Get operator name
    QString name() const;
    //! Get operator availability
    Availability availability() const;
    //! Get operator availability in human readable format
    QString availabilityName() const;
    //! Get operator supported band information
    Band band() const;
    //! Get operator supported band information in human readable format
    QString bandName() const;
    //! Get operator preference
    Preference preference() const;
    //! Get operator preference in human readable format
    QString preferenceName() const;
    //! Is UMTS available with this operator
    bool umts() const;

private:
    QExplicitlySharedDataPointer<AvailableOperatorData> d;
};

QDebug operator<<(QDebug dbg, const AvailableOperator &d);

//! Network registration.
/*!
 * This class implements network registration support. It provides
 * information on current registration status of the device, and
 * offers methods for available network operator scanning, and manual and
 * automatic network registration.
 */
class CELLULAR_QT_EXPORT NetworkRegistration : public QObject
{

    Q_OBJECT
    Q_PROPERTY(Status status READ status)
    Q_PROPERTY(RejectCode rejectCode READ rejectCode)
    Q_PROPERTY(Mode mode READ mode)
    Q_PROPERTY(bool valid READ isValid)
    Q_ENUMS(Status)
    Q_ENUMS(RejectCode)
    Q_ENUMS(Mode)

public:
    NetworkRegistration(QObject *parent=0);
    ~NetworkRegistration();

    //! Network registration status.
    enum Status {
        UnknownStatus = -1, //!< Registration status is unknown
        AtHome,             //!< Registered home
        Roaming,            //!< Roaming
        Offline,            //!< Device is offline
        Searching,          //!< Searching for networks
        NoSim,              //!< Offline; no SIM present
        Rejected,           //!< Offline; network rejected the registration
        PowerOff,           //!< Offline and powered off
        PowerSave,          //!< Offline and in power save mode
        NoCoverage          //!< Offline due to no coverage
    };

    //! Network registration success and failure codes.
    enum RejectCode {
        Success = -1,               //!< Registration succeeded.
        IMSIUnknownInHLR,       //!< IMSI unknown in HLR.
        IllegalMS ,              //!< Illegal MS.
        IMSIUnknownInVLR,       //!< IMSI Unknown in VLR.
        IMEINotAccepted ,        //!< IMEI not accepted.
        IllegalME ,              //!< Illegal ME.
        GPRSServicesNotAllowed , //!< GPRS services not allowed.
        NonGPRSServicesNotAllowed , //! GPRS and non-GPRS services not allowed.
        IdentityCannotBeDerived , //!< MS identity cannot be derived by the network.
        ImplicitlyDetached ,     //!< Implicitly detached.
        PLMNNotAllowed ,         //!< PLMN not allowed.
        LocationAreaNotAllowed , //!< Location area not allowed.
        RoamingNotAllowed ,      //!< Roaming in this location area not allowed.
        ServicesNotAllowedInThisPLMN , //!< GPRS services not allowed in this PLMN.
        NoCellsInLocationArea ,  //!< No suitable cells in this location area.
        TemporarilyNotReachable , //!< MSC temporarily not reachable.
        NetworkFailure ,         //!< Network failure.
        MACFailure ,             //!< MAC failure.
        SynchFailure ,           //!< Syncronization failure.
        Congestion ,             //!< Congestion.
        AuthNotAcceptable ,      //!< GSM authentication not acceptable.
        OptionNotSupported ,    //!< Service option not supported.
        OptionNotSubscribed ,    //!< Service option not subscribed.
        OptionOutOfOrder ,       //!< Service option temporarily out or order.
        RetryUponNewCell ,       //!< Retry upon entry into a new cell.
        BadMessage ,             //!< Semantically incorrect message.
        InvalidInformation ,     //!< Invalid mandatory information.
        NoSuchMessage ,          //!< Message type either non-existent or not implemented.
        ConditionalIEError ,    //!< Conditional IE error.
        IncompatibleMessage,   //!< Message type incompatible with protocol state.
        UnspecifiedError ,      //!< Unspecified protocol error.
     };

    //! Network selection mode.
    enum Mode
    {
	UnknownMode = -1, //!< Network access mode is unknown
	Manual,           //!< Manual network access selection
	Automatic,        //!< Automatic network access selection
	ForcedAutomatic   //!< Manual network access selection is not allowed
    };

    //! Get registration status
    /*!
     * This method returns the current registration status.
     *
     * \return Current registration status, as
     * NetworkRegistration::Status.
     */
    Status status() const;

    //! Get registration reject code
    /*!
     * This method returns the current registration reject code, if status()
     * has returned NetworkRegistration::Rejected.
     *
     * \return Current registration reject code, as
     * NetworkRegistration::RejectCode.
     */
    RejectCode rejectCode() const;

    //! Get access selection mode
    /*!
     * This method returns the current network access selection
     * mode.
     *
     * NetworkRegistration::Manual means that the user is making the
     * decision on which network to use.
     * NetworkRegistration::Automatic means the cellular modem
     * makes the decision.
     *
     * NetworkRegistration::ForcedAutomatic is a special case. It
     * indicates that the home operator policy disallows manual
     * selection, and therefore the selection mode is forced into
     * automatic. A user interface should in this case grey out any
     * manual selection widgets.
     *
     * \return The current network selection mode, as
     * NetworkRegistration::Mode
     */
    Mode mode() const;

    //! Get the latest query results
    /*!
     * This method returns available operators as a result of the
     * latest available operator query.
     *
     * \return Currently available operator list, or an empty list if
     * no scan has been performed.
     */
    QList<AvailableOperator> availableOperators() const;

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service was not responding and the properties are
     * not available. This is an indication something is seriously
     * wrong, and should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover. Perhaps deleting the class instance, and
     * trying again after a while might help.
     */
    bool isValid() const;

public Q_SLOTS:

    //! Automatically select any available operator
    /*!
     * This method allows selecting a network operator
     * automatically. The registration operation may take up to 90
     * seconds. The registration results are returned via the
     * selectionCompleted() signal.
     */
    //! @credential %Cellular resource token required
    void selectOperator();

    //! Select an operator based on MNC/MCC code
    /*!
     * This method allows selecting a named network operator. The
     * registration operation may take up to 90 seconds. The
     * registration results are returned via the selectionCompleted()
     * signal.
     *
     * \param mnc Network code of the operator to register with
     * \param mcc Country code of the operator to register with
     */
    //! @credential %Cellular resource token required
    void selectOperator(const QString &mnc, const QString &mcc);

    //! Select an operator from an AvailableOperator object
    /*!
     * This method allows selecting an AvailableOperator object. The
     * registration operation may take up to 90 seconds. The
     * registration results are returned via the selectionCompleted()
     * signal.
     *
     * \param oper An AvailableOperator instance to register with
     */
    //! @credential %Cellular resource token required
    void selectOperator(const AvailableOperator &oper);

    //! Select an operator with index of current scan results
    /*!
     * This method allows selecting an AvailableOperator object. The
     * registration operation may take up to 90 seconds. The
     * registration results are returned via the
     * selectOperatorSuccess() or selectOperatorFailed() signals.
     *
     * \param index An index to the latest available operators scan
     * results.
     */
    //! @credential %Cellular resource token required
    void selectOperator(int index);

    //! Cancel an ongoing query for selecting an operator
    /*!
     * This method can be used to cancel an ongoing query to
     * select an operator (except automatic operator selection which is
     * not possible to cancel). The cancel operation should be quick and
     * result will be emitted via the selectOperatorCanceled() signal.
     * Also, selectionCompleted() signal should return a failure simultaneously.
     */
    //! @credential %Cellular resource token required
    void cancelSelectOperator();

    //! Query for available networks
    /*!
     * This method initiates a scan for available networks. The
     * operation may take up to 60 seconds. The scan results will be
     * emitted via the availableOperators() signal.
     */
    //! @credential %Cellular resource token required
    void queryAvailableOperators();

    //! Cancel an ongoing query for available networks
    /*!
     * This method can be used to cancel an ongoing query for
     * available networks. The cancel operation should be quick and
     * result will be emitted via the availableOperatorsCanceled() signal.
     * Also, availableOperators() signal should return a failure simultaneously.
     */
    //! @credential %Cellular resource token required
    void cancelQueryAvailableOperators();

    //! Scan for availble operators
    /*!
     * This method initiates a scan for available networks. The
     * operation may take up to 60 seconds. The scan results will be
     * emitted via the operatorsSuccess() and operatorsFailed()
     * signala. */
    //! @credential %Cellular resource token required
    void scanOperators();

    //! Reset object internal state
    /*!
     * This method will reset the internal state of this class
     * instance, canceling any pending registrations, operator scans
     * and clear any pending scan results.
     */
    //! @credential %Cellular resource token required
    void reset();

Q_SIGNALS:
    //! Registration status changed
    /*!
     * This signal is emitted then the registration status has
     * changed.
     *
     * \param status New registration status
     */
    void statusChanged(int status);

    //! Registration status changed, with reject code
    /*!
     * This signal is emitted then the registration status has
     * changed. It also contains the rejection code, if SIM was
     * rejected by the network.
     *
     * \param status New registration status
     * \param rejectCode Registration reject code.
     */
    void statusWithRejectCodeChanged(int status, int rejectCode);

    //! Access selection mode changed
    /*!
     * This signal is emitted when the network access selection mode
     * has changed.
     *
     * \param mode New access selection mode
     */
    void modeChanged(int mode);

    //! Asynchronous response to available network scan
    /*!
     * This signal is emitted when a scan for available operators is
     * completed, and results are available.
     *
     * \param success Set to true if the scan completed successfully
     * and false otherwise.
     * \param operators Scan results as a list of AvailableOperator objects
     * \param reason Error reason if the scan completed unsuccessfully
     */
    void availableOperators(bool success, const QList<AvailableOperator> &operators,
			    const QString &reason);


    //! Asynchronous response to canceling available network scan
    /*! 
     * This signal is emitted when available network scan has been canceled.
     *
     * \param success Set to true if the canceling completed successfully
     * and false otherwise.
     * \param reason Error reason if the canceling completed unsuccessfully
     */
    void availableOperatorsCanceled(bool success, const QString &reason);

    //! Deprecated
    void selectionCompleted(bool success, const QString &reason);

    //! Asynchronous response to network selection
    /*!
     * This signal is emitted when the network registration operation
     * has completed.
     *
     * \param code Error reason as a NetworkRegistration::RejectCode.
     */
    void selectionCompleted(int code);

    //! Asynchronous response to canceling operator selection
    /*! 
     * This signal is emitted when operator selection has been cancelled.
     *
     * \param success Set to true if the canceling completed successfully
     * and false otherwise.
     * \param reason Error reason if the canceling completed unsuccessfully
     */
    void selectOperatorCanceled(bool success, const QString &reason);

    //! Asynchronous success response to network selection
    /*!
     * This signal is emitted when the network registration operation
     * has completed successfully.
     */
    void selectionSuccess();

    //! Asynchronous failure response to network selection
    /*!
     * This signal is emitted when the network registration operation
     * has completed unsuccessfully.
     */
    void selectionFailure();

    //! Asynchronous response for when selection gets canceled
    /*!
     * This signal is emitted when the network registration operation
     * gets canceled.
     */
    void selectionCanceled();

    //! Asynchronous success response to operator scanning.
    /*!
     * This signal is emitted when the network operator scan has
     * completed successfully.
     *
     * The list of available operators is available with the
     * availableOperators() method.
     */
    void operatorsSuccess();

    //! Asynchronous failure response to operator scanning.
    /*!
     * This signal is emitted when the network operator scan has
     * completed unsuccessfully.
     */
    void operatorsFailure();

    //! Asynchronous response when scanning gets canceled
    /*!
     * This signal is emitted when the network operator scan gets
     * canceled.
     */
    void operatorsCanceled();

private:
    NetworkRegistrationPrivate *d;
};

}

#endif  /* !NETWORKREGISTRATION_H */
