/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef NETWORKTIME_H
#define NETWORKTIME_H

#include <QtCore/QObject>
#include <QtCore/QDateTime>
#include <QSharedData>
#include <QtCore/QExplicitlySharedDataPointer>
#include <time.h>

#include "libcellular-qt_global.h"

namespace Cellular
{

class NetworkTimeData : public QSharedData
{
public:
    NetworkTimeData() : dateTime(QDateTime()), offsetFromUtc(0),
	daylightAdjustment(-1), isValid(false) {

	timestamp.tv_sec = 0;
	timestamp.tv_nsec = 0;

	mnc.clear();
	mcc.clear();
    }

    NetworkTimeData(const NetworkTimeData &other) : QSharedData(other),
	dateTime(other.dateTime), offsetFromUtc(other.offsetFromUtc),
	daylightAdjustment(other.daylightAdjustment), isValid(other.isValid),
	mnc(other.mnc),	mcc(other.mcc) {

	timestamp.tv_sec = other.timestamp.tv_sec;
	timestamp.tv_nsec = other.timestamp.tv_nsec;
    }

    ~NetworkTimeData() { }

    QDateTime dateTime;
    int offsetFromUtc;
    int daylightAdjustment;
    bool isValid;
    QString mnc;
    QString mcc;
    struct timespec timestamp;
};

//! Network time information
/*!
 * This class provides access to network time information, as it is
 * received from the network. A NetworkTimeInfo object is invalid if
 * the time information is not available. If the UTC time is not
 * available, the QDateTime() returned by dateTime() is set to
 * invalid. If the daylight savings time adjustment is not available,
 * it is set to -1.
*/
class NetworkTimeInfo
{

public:

    NetworkTimeInfo() {	d = new NetworkTimeData; }

    NetworkTimeInfo(const QDateTime &dateTime, const int daylightAdjustment,
		    const int offsetFromUtc, const qlonglong timestampSeconds,
		    const qlonglong timestampNanoSeconds,
		    const QString &mnc,
		    const QString &mcc)
	{
	    d = new NetworkTimeData;
	    d->dateTime = dateTime;
	    d->daylightAdjustment = daylightAdjustment;
	    d->offsetFromUtc = offsetFromUtc;
	    d->timestamp.tv_sec = timestampSeconds;
	    d->timestamp.tv_nsec = timestampNanoSeconds;
	    d->mnc = mnc;
	    d->mcc = mcc;
	    d->isValid = true;
	}

    NetworkTimeInfo(const NetworkTimeInfo &other)
	: d (other.d) { }
    ~NetworkTimeInfo() { }

    //! Get the current time information.
    /*!
     * This method returns the time information as it was reported by
     * the network, as a QDateTime instance. This QDateTime has its
     * Qt::TimeSpec set to Qt::UTC. If no time information was
     * received from the network, this information is not available,
     * and hence this QDateTime instance will be invalid. Note: to
     * calculate the current time, this QDateTime needs to be adjusted
     * using the values returned in timestamp().
     *
     * \return This method returns a QDateTime object.
     */
    QDateTime dateTime() const { return d->dateTime; }

    //! Get the current timezone offset.
    /*!
     * This method returns the timezone offset in seconds from UTC, as
     * reported by the network. Negative value offsets denote time
     * zones west of UTC, and positive values east of UTC.
     *
     * This value includes the daylight savings time adjustment
     * returned by daylightAdjustment(). This value is guaranteed to
     * be in multples of 15 minute intervals, accounting for time
     * zones that use such offsets.
     *
     * For more detailed description of time zone, \see
     * http://en.wikipedia.org/wiki/Timezone.
     *
     * \return This method returns the timezone offset from UTC in
     * seconds; negative values are west of UTC, positive are east of
     * UTC.
     */
    int offsetFromUtc() const { return d->offsetFromUtc; }

    //! Get the currently active daylight savings adjustment.
    /*!
     * This method returns the current daylight adjustment as reported
     * by the network. The value returned is in hours, or -1 if the
     * current adjustment is unknown. The standards allows for 0, 1,
     * or 2 hour adjustment, but likely only 0, and 1 are in actual
     * use in the world.
     *
     * \return This method returns the current Daylight Savings (DST)
     * adjustment in hours, or -1 if the value is not available.
     */
    int daylightAdjustment() const {
	return d->daylightAdjustment;
    }

    //! Get the timestamp of last received time information.
    /*!
     * This method returns in nanosecond-precision timestamp of the
     * last received network time information. This timestamp is based
     * on the system monotonic clock.
     *
     * \return Timestamp of receiving the network time indication.
     */
    struct timespec *timestamp() const {
	return &d->timestamp;
    }

    //! Get the network operator code.
    /*!
     * This method returns the network operator information for the
     * network that sent the time information.
     *
     * There is a reason that the return value is a string and not an
     * unsigned. The network code can be 2 or 3 digits long, and
     * unfortunately leading zeros are not silent. I.e., an MNC of
     * '005' is not the same as '05'. Some regions use leading zeros
     * while others don't.
     *
     * \see http://en.wikipedia.org/wiki/Mobile_Network_Code
     *
     * \return Mobile Network Code (MNC) as string, or an empty string if
     * the data is not available.
     */
    QString mnc() const { return d->mnc; }

    //! Get the network country code.
    /*!
     * This method returns the network country code information for
     * the network that sent the time information. The country code is
     * a string of three decimal digits, with no leading zeros.
     *
     * \see http://en.wikipedia.org/wiki/Mobile_Country_Code
     *
     * \return Mobile Country Code (MCC) as string, or an empty string
     * if the data is not available.
     */
    QString mcc() const { return d->mcc; }

    //! Check validity of class instance.
    /*!
     * This method returns the validity of the network time
     * information. If false, the information is not available, i.e.,
     * the network doesn't support the NITZ extension, or the cellular
     * services backend was not able to produce the information.
     *
     * \return This method returns true if the data is valid,
     * otherwise false.
     */
    bool isValid() const { return d->isValid; }

private:
    QExplicitlySharedDataPointer<NetworkTimeData> d;
};

QDebug operator<<(QDebug dbg, const NetworkTimeInfo &d);

class NetworkTimePrivate;

//! Network time information
/*!
 * The network time exposed by this API is stateless; it merely
 * mirrors the most recent time information received from the network.
 *
 * Time information is received from the network via Network Identity
 * and Timezone (NITZ) extension to GSM. A NITZ indication is a single
 * PDU received from the network. It contains the current UTC time,
 * the current timezone's offset to UTC and the current daylight
 * savings setting. It's also possible that NITZ only carries the
 * timezone offset and DST setting, or only the timezone offset.
 *
 * \sa http://en.wikipedia.org/wiki/NITZ
*/
class CELLULAR_QT_EXPORT NetworkTime : public QObject
{

    Q_OBJECT
    Q_PROPERTY(NetworkTimeInfo timeInfo READ timeInfo)
    Q_PROPERTY(bool valid READ isValid)

public:
    NetworkTime(QObject *parent=0);
    ~NetworkTime();

    //! Get the current date and time.
    /*!
     * This method returns a NetworkTimeInfo instance. Note that this
     * method blocks. For a non-blocking, asynchronous version, use
     * queryTimeInfo().
     *
     * \returns This method returns a NeteorkTimeInfo instance.
     */
    NetworkTimeInfo timeInfo() const;

    //! Initiate a query of current date and time.
    /*!
     * This method initiates an asynchronous query for network
     * time. It returns immediately, and the results are delivered in
     * a timeInfoQueryCompleted() signal.
     */
    void queryTimeInfo();

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service is not responding and the properties are
     * not available. This should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover. Trying again later might help.
     */
    bool isValid() const;

Q_SIGNALS:
    //! Date and time settings have changed
    void timeInfoChanged(const NetworkTimeInfo &timeInfo);

    //! Query for date and time settings completed
    void timeInfoQueryCompleted(const NetworkTimeInfo &timeInfo);

private:
    NetworkTimePrivate *d;
};

}
#endif  /* !NETWORKTIME_H */
