/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef RADIOACCESS_H
#define RADIOACCESS_H

#include <QtCore/QObject>
#include <QtCore/QFlags>

#include "libcellular-qt_global.h"

namespace Cellular
{

class RadioAccessPrivate;

//! Radio access.
/*!
 * This class provides details of the cellular radio access. It
 * provides information on the currently used Radio Access Technology
 * (RAT), RAT selection mode, available data access technology, as
 * well as details about the state of the radio access:
 *
 * - Radio link idle or busy?
 * - Data access available or in suspended state
 * - High speed packet downlink allocated
 * - High speed packet uplink allocated
 */
class CELLULAR_QT_EXPORT RadioAccess : public QObject
{

    Q_OBJECT
    Q_PROPERTY(Technology technology READ technology)
    Q_PROPERTY(Mode mode READ mode WRITE setMode)
    Q_PROPERTY(uint channel READ channel)
    Q_PROPERTY(bool dataAvailability READ isDataAvailable)
    Q_PROPERTY(bool valid READ isValid)
    Q_ENUMS(Technology)
    Q_ENUMS(Mode)
    Q_FLAGS(Flags flags READ state)

public:
    enum Option
    {
	EnableRadioState = 0x1 << 0,  //!< Enable async \a radio flags indications.
    };
    Q_DECLARE_FLAGS(Options, Option);

    RadioAccess(QObject *parent=0);
    RadioAccess(Options options, QObject *parent=0);
    ~RadioAccess();

    //! Current radio access and data access technology
    enum Technology
    {
	None = -1,       //!< Current access technology is unknown.
	TwoG,            //!< GPRS, or 2G is currently available.
	TwoPointFiveG,   //!< EGPRS, or 2.5G is currently available.
	ThreeG,          //!< UMTS, or 3G is currently available.
	ThreePointFiveG, //!< HSPA, or 3.5G is currently available.
    };

    //! Current selection mode for radio access
    enum Mode
    {
	Unknown = -1, //!< Radio access selection mode unknown.
	DualMode,     /*!< Dual mode access selection in use; 2G or 3G
		       * will be selected based on availability. */
	OnlyTwoG,     //!< GSM only access selection mode in use.
	OnlyThreeG,   /*!< UMTS only access selection mode in use. If
		       * 3G is unavailable, the device will deregister
		       * completely. */
	ForcedDualMode, /*!< Dual mode access selection is in use.
			 * Changing the selection mode is not possible. */
    };

    //! Radio access state
    enum State
    {
	Idle = 0x1 << 0,              /*!< Radio access is idle. Corresponds to
				       * the CELL_DCH link state. */
	HighSpeedDownlink = 0x1 << 1, //!< High speed downlink (HSDPA channel) allocated.
	HighSpeedUplink = 0x1 << 2,   //!< High speed uplink (HSUPA channel) allocated.
    };
    Q_DECLARE_FLAGS(Flags, State)

    //! Get current access mode
    /*!
     * This method returns the current Radio Access Technology (RAT)
     * selection mode.
     *
     * \return Current radio access technology selection mode as RadioAccess::Mode
     */
    Mode mode() const;

    //! Get current access technology
    /*!
     * This method returns the currently used Radio Access Technology
     * (RAT).
     *
     * \return Current radio access technology as RadioAccess::Technology
     */
    Technology technology() const;

    //! Get current radio access state
    /*!
     * This method returns a set of flags for the current radio access
     * state.
     *
     * \return Current radio access state as RadioAccess::Flags
     */ 
    Flags state() const;

    //! Get current channel
    /*!
     * This method returns the current UTRA Absolute Radio Frequency
     * Channel Number (UARCFN). A valid range for the channel() is
     * 0...16383; if the channel is currently unavailable, -1 is
     * returned. The channel will be unavailable when registered in a
     * 2G only cell.
     *
     * \return Current UTRA channel number, or -1 if it is unavailable.
     */
    int channel() const;

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service was not responding and the properties are
     * not available. This is an indication something is seriously
     * wrong, and should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover, except to try again later.
     */
    bool isValid() const;

    //! Check the current options regarding indication granularity.
    /*!
     * This method allows querying the currently enabled
     * RadioAccess::Options, which correspond to the currently enabled
     * granularity for radio idle and HSDPA/HSUPA channel allocations
     * from the cellular backend.
     *
     * \return The current configuration options for async indications
     */
    Options options() const;

    //! Set the current options.
    /*!
     * This method allows configuring the RadioAccess::Options, which
     * correspond to the currently enabled granularity for radio idle
     * and HSDPA/HSUPA channel allocation indications from the
     * cellular backend.
     *
     * \param options New configure options
     */
    void setOptions(Options options);

    //! Check current data availability.
    /*!
     * This method is used to check the current availability of GPRS
     * data access. The access can be temporarily disabled, e.g.,
     * during a voice call in some GSM networks.
     *
     * \return availability Current availability status
     */
    bool isDataAvailable() const;

public Q_SLOTS:
    //! Set radio access selection mode
    /*!
     * This method allows setting the current radio access selection
     * mode. Calling this will cause the selection mode to change,
     * which will be reported with the modeChanged() signal. Trying to
     * set mode to the current selection mode does nothing.
     *
     * \param mode The desired radio access mode
     */
    //! @credential %Cellular resource token required
    void setMode(RadioAccess::Mode mode);

Q_SIGNALS:
    //! Radio access status changed
    /*!
     * This signal is emitted when any of the radio access flags have
     * changed.
     *
     * \param flags Current radio access state
     */
    void statusChanged(RadioAccess::Flags flags);

    //! Radio access selection mode changed
    /*!
     * This signal is emitted when the radio access selection mode has
     * changed.
     *
     * \param mode New radio access selection mode
     */
    void modeChanged(int mode);

    //! Setting radio access selection mode failed
    /*!
     * This signal is emitted when changing the radio access selection
     * mode has failed.
     *
     * \param cause Error message
     */
    void modeChangeFailed(const QString &cause);

    //! Radio access technology changed
    /*!
     * This signal is emitted when the currently used radio access
     * technology has changed.
     *
     * \param technology New radio access technology
     */
    void technologyChanged(int technology);

    //! UTRA channel changed
    /*!
     * This signal is emitted when the current UARFCN has changed.
     *
     * \param channel New UARFCN value
     */
    void channelChanged(int channel);

    //! Data availability changed
    /*!
     * This signal is emitted when the data availability status
     * changes.
     *
     * \param availability The updated availability value
     */
    void dataAvailabilityChanged(bool available);

private:
    RadioAccessPrivate *d;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(RadioAccess::Flags)

}

#endif  /* !RADIOACCESS_H */
