/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009-2010  Nokia Corporation. All rights reserved.
 */

#ifndef SIGNALSTRENGTH_H
#define SIGNALSTRENGTH_H

#include <QtCore/QObject>

#include "libcellular-qt_global.h"

namespace Cellular
{

class SignalStrengthPrivate;

//! Network signal strength.
/*!
 * This class is used to query and receive notifications of cellular
 * Received Signal Strength Indications (RSSI) in GSM, or Received
 * Signal Code Power (RSCP) when using UMTS.
 *
 * Three types of values are provided:
 *
 * - \a bars; a discrete value between 0...5 corresponding to the number
 *   of bars to display in a UI.
 * - \a percent; a discrete value between 0...100% of maximum power.
 * - \a dbm; an RSSI power ratio in decibels to reference level of one
 *   milliwatt.
 *
 * Often it is not useful to follow changes in dBm or percentage of
 * maximum, if one merely wants to display zero to five bars in a
 * UI. By defining appropriate flags in the constructor, this class
 * can be made to only listen to changes in certain types of signal
 * level values while ignoring absolute chenges.
 */
class CELLULAR_QT_EXPORT SignalStrength : public QObject
{

    Q_OBJECT
    Q_PROPERTY(int bars READ bars)
    Q_PROPERTY(int percent READ percent)
    Q_PROPERTY(int dbm READ dbm)
    Q_PROPERTY(bool valid READ isValid)
    Q_PROPERTY(Options options READ options WRITE setOptions)

public:
    //! Signal strength indicator granularity
    /*!
     * These flags are used to control how this class interacts with
     * the cellular backend. They control the granularity with which
     * signal strength changes are communicated from the backend.
     *
     * For example, by defining only \a SignalStrength::Bars in the
     * class constructor, the class will not connect to the backend's
     * percentage/dBm indications, nor will it emit the
     * percentChanged() and dbmChanged() signals. The property getters
     * will still work.
     *
     * This can help reduce the number of wake ups that this class
     * instance needs to perform. By default, no async changes are
     * provided.
     */
    enum Option
    {
	EnableBars = 0x1 << 0,     //!< Enable async \a bars indications.
	EnablePercent = 0x1 << 1,  //!< Enable async \a percent indications.
	EnableDecibels = 0x1 << 2, //!< Enable async \a dbm indications.
    };
    Q_DECLARE_FLAGS(Options, Option);

    SignalStrength(QObject *parent=0);
    SignalStrength(Options flags, QObject *parent=0);
    ~SignalStrength();

    //! Get the current level in bars.
    /*!
     * This method returns the current signal strength level in bars
     * in the range 0...5. These bars can easily be mapped into a UI
     * that has these same six states.
     *
     * The backend automatically adjust the algorithm with which real
     * signal power measurements are mapped into bars. For instance,
     * some operator variants will map slightly different power levels
     * to maximum bars, or denote slightly lower power values to one
     * bar available.
     *
     * If the UI has a different scale for bars, it is advisable to
     * use the percent() or dbm() properties and corresponding
     * signals, and perform any custom mapping to UI bars.
     */
    int bars() const; 

    //! Get the current level in percentages.
    /*!
     * This method returns the current received signal strength level
     * in percentages. This value is in the range of 0...100, or -1 if
     * the device is currently not in service.
     *
     * \return Current signal strength level in percentages.
     */
    int percent() const;

    //! Get the current level in dBm.
    /*!
     * This method returns the current received signal strength level
     * as the RSSI power ratio in decibels to 1mW. This value is in
     * the range of -1...-123, or 0 if the device is not in service.
     *
     * \return Current signal strength level in dBm.
     */
    int dbm() const;

    //! Check validity of class instance.
    /*!
     * This method can be used to check validity of the class
     * instance.
     *
     * If the isValid() method returns false, it typically means that
     * the backend service was not responding and the properties are
     * not available. This is an indication something is seriously
     * wrong, and should never happen.
     *
     * If isValid() indeed returns false, there is little that can be
     * done to recover. Perhaps deleting the class instance, and
     * trying again after a while might help.
     */
    bool isValid() const;

    //! Check the current flags. regarding indication granularity.
    /*!
     * This method allows querying the currently enabled
     * SignalStrength::Options, which correspond to the currently
     * enabled granularity for signal strength level indications from
     * the cellular backend.
     */
    Options options() const;

    //! Set the current flags.
    /*!
     * This method allows configuring the SignalStrength::Options,
     * which correspond to the currently enabled granularity for
     * signal strength level indications from the cellular backend.
     */
    void setOptions(Options flags);

Q_SIGNALS:
    // Signal strength bars changed
    /*!
     * This signal is emitted when the signal strength level has
     * changed. The value is in bars, with range of 0...5, or -1 if
     * the device has dropped out from service.
     *
     * \param bars New signal strength level in bars.
     */
    void barsChanged(int bars);

    // Signal strength percentage changed
    /*!
     * This signal is emitted when the signal strength level has
     * changed. The value is in percentage from maximum, with range of
     * 0...100, or -1 if the device has dropped out from service.
     *
     * \param bars New signal strength level in percentages.
     */
    void percentChanged(int percent);

    // Signal strength dbm changed
    /*!
     * This signal is emitted when the signal strength level has
     * changed. The value is in dBm, with range of -1...-123, or 0 if
     * the device has dropped out from service.
     *
     * \param bars New signal strength level in dBm.
     */
    void dbmChanged(int dbm);

private:
    SignalStrengthPrivate *d;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(SignalStrength::Options)

}
#endif  /* !SIGNALSTRENGTH_H */
