/*
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 */

#ifndef SIM_H
#define SIM_H

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QStringList>

#include "libcellular-qt_global.h"

namespace Cellular {

//! SIM namespace
namespace SIM {

class SIMStatusPrivate;
class SIMIdentityPrivate;
class SIMSecurityPrivate;

//! SIM errors
enum SIMError
{
    NoError = 0,            /*!< Returned when no error happened and
			     * operation completed successfully */
    BusCommunicationError,  /*!< Returned when a D-Bus communication
			     * error with the Cellular Services daemon
			     * occured */
    InvalidParameter,       /*!< Returned when provided parameters
			     * were invalid. */
    ModemNotReady,          /*!< Cellular modem was not yet in ready
			     * state. */
    ServiceBusy,            /*!< Too many pending requests towards the
			     * modem; the service is unable to
			     * complete the given task. */
    Modem,                  /*!< Modem internal error. */
    DataNotAvailable,       /*!< Requested data was not available on
			     * the SIM. */
    WrongPassword,          /*!< Given password was incorrect. */
    PINDisabled,            /*!< PIN code is disabled. */
    CodeBlocked,            /*!< Code is blocked. */
    NoSIM,                  /*!< No SIM card present in the device. */
    SIMRejected,            /*!< The SIM card present in the device
			     * was rejected. */
    SecurityCodeRequired,   /*!< Security code is required for the
			     * action to complete. */
    Unknown                 /*!< Unknown error. */
};

//! SIM status.
/*!
 * This class provides access to SIM status and status change
 * notification.
 *
 * Note that all methods are asynchronous and have a
 * corresponding completion/result signal.
*/
class CELLULAR_QT_EXPORT SIMStatus: public QObject
{
    Q_OBJECT

public:
    //! Possible SIM status list
    enum Status
    {
	UnknownStatus = -1, /*!< Unknown status. */
	Ok,                 /*!< SIM card is ok. */
	NoSIM,              /*!< No SIM card available. */
	PermanentlyBlocked, /*!< SIM card is permanently blocked. */
	NotReady,           /*!< SIM card is not ready. */
	PINRequired,        /*!< PIN code is required to complete the
			     * request. */
	PUKRequired,        /*!< PUK code is required to complete the
			     * request. */
	Rejected,           /*!< SIM card was rejected. */
	SIMLockRejected     /*!< SIM lock was rejected. */
    };

    SIMStatus(QObject *parent=0);
    ~SIMStatus();

    //! Check validity of class instance.
    bool isValid() const;

public Q_SLOTS: // METHODS
    //! Request current status/state of SIM
    void status();

Q_SIGNALS: // SIGNALS
    //! Emitted when SIM status changes
    void statusChanged(SIMStatus::Status status);

    //! Emitted when status() call completes
    void statusComplete(SIMStatus::Status status, SIMError error);

private:
    SIMStatusPrivate *d;
};

//! SIM identity.
/*!
 * Provides access to information stored in the SIM.
 *
 * Note that all methods are asynchronous and have a corresponding
 * completion/result signal.
*/
class CELLULAR_QT_EXPORT SIMIdentity: public QObject
{
    Q_OBJECT

public:
    SIMIdentity(QObject *parent=0);
    ~SIMIdentity();

    //! Check validity of class instance.
    bool isValid() const;

public Q_SLOTS: // METHODS
    //! Request MCC and MNC values of home operator.
    void hplmn();

    //! Request IMSI value
    void imsi();

    //! Request service provider name
    void serviceProviderName();

    //! Request list of preferred languages 
    /*! This method may be called before PIN has been verified */
    void languages();

    //!  Returns ICCID of the SIM
    /*! This method may be called before PIN has been verified */
    void iccid();

    //!  Performs SIM authentication
    /*! \param rand Array of random bytes, must be exactly 16 bytes long */
    void auth(QByteArray rand);

Q_SIGNALS: // SIGNALS
    //! Emitted when hplmn() call completes
    void hplmnComplete(QString mcc, QString mnc, SIMError error);

    //! Emitted when imsi() call completes
    /*! \param imsi Current IMSI (EF-IMSI) coded as ASCII digits, e.g.
      "244070100400103" */
    void imsiComplete(QString imsi, SIMError error);

    //! Emitted when serviceProviderName() call completes
    /*! \param spn Service Provider Name (EF-SPN), e.g. "GEMPLUS" */
    void serviceProviderNameComplete(QString spn, SIMError error);

    //! Emitted when languages() call completes
    /*! \param languages two letter language codes (ISO 639-1 Code) like "de",
      "en","it","fr","es","nl","sv","da" */
    void languagesComplete(QStringList languages, SIMError error);

    //! Emitted when iccid() call completes
    /*! \param iccid ICCID of the SIM coded as ASCII digits,
      e.g. "8912690010119026229" */
    void iccidComplete(QString iccid, SIMError error);

    //! Emitted when auth() call completes
    /* \param res maximum of 16 bytes, can be less
       \param cipheringKey 8 bytes
       \param eapCipheringKey 16 bytes
       \param eapIntegrityKey 16 bytes */
    void authComplete(QByteArray res, QByteArray cipheringKey, QByteArray eapCipheringKey, QByteArray eapIntegrityKey, SIMError error);

private:
    SIMIdentityPrivate *d;
};

//! SIM security.
/*!
 * Provides SIM security related features, such as PIN code
 * verification.
 *
 * Note that all methods are asynchronous and have a corresponding
 * completion/result signal.
*/
class CELLULAR_QT_EXPORT SIMSecurity: public QObject
{
    Q_OBJECT

public:
    //! Possible PIN code types
    enum PINType {
	PIN, /*!< PIN */
	PIN2 /*!< PIN2 */
    };

    //! Possible PUK code types
    enum PUKType {
	PUK, /*!< PUK */
	PUK2 /*!< PUK2 */
    };

    //! PIN or PIN2 query state: enabled, disabled or unknown
    enum PINQuery {
	UnknownState = -1, /*!< PIN/PIN2 state is unknown. */
	Enabled,           /*!< PIN/PIN2 is enabled. */
	Disabled           /*!< PIN/PIN2 is disabled. */
    };

public:
    SIMSecurity(QObject *parent=0);
    ~SIMSecurity();

    //! Check validity of class instance.
    bool isValid() const;

public Q_SLOTS: // METHODS

    //! Changes PIN or PIN2 code
    //! @credential %Cellular resource token required
    void changePIN(PINType type, const QString &oldCode, const QString &newCode);

    //! Disables PIN or PIN2 code protection of SIM
    //! @credential %Cellular resource token required
    void disablePINQuery(PINType type, const QString &code);

    //! Enables PIN or PIN2 code protection of SIM.
    //! @credential %Cellular resource token required
    void enablePINQuery(PINType type, const QString &code);

    //! Requests PIN or PIN2 code protection state
    void pinQueryState(PINType type);

    //! Verifies PIN or PIN2 code. 
    /*! Verifying the PIN code opens the SIM,
	verifying PIN2 code opens access to PIN2 protected services, for example 
	FDN phonebook. */
    //! @credential %Cellular resource token required
    void verifyPIN(PINType type, const QString &code);

    /*! Unblocks PIN or PIN2 code by verifying PUK or PUK2 code respectively, sets
      the new PIN or PIN2 code. */
    //! @credential %Cellular resource token required
    void verifyPUK(PUKType type, const QString &code, const QString &newPinCode);

   //! Requests the number of PIN or PIN2 verification attempts left
    void pinAttemptsLeft(PINType type);

   //! Requests the number of PUK or PUK2 verification attempts left
    void pukAttemptsLeft(PUKType type);

Q_SIGNALS: // SIGNALS
    //! Emitted when verification of PIN or PIN2 code is needed.
    void verifyPINRequired(SIMSecurity::PINType type);

    //! Emitted when verification of PUK or PUK2 code is needed.
    void verifyPUKRequired(SIMSecurity::PUKType type);

    //! Emitted when changePIN() call completes
    void changePINComplete(bool success, SIMError error);

    //! Emitted when disablePINQuery() call completes
    void disablePINQueryComplete(SIMError error);

    //! Emitted when enablePINQuery() call completes
    void enablePINQueryComplete(SIMError error);

    //! Emitted when pinQueryState() call completes
    void pinQueryStateComplete(SIMSecurity::PINQuery state, SIMError error);

    //! Emitted when verifyPIN() call completes
    void verifyPINComplete(bool success, SIMError error);

    //! Emitted when verifyPUK() call completes
    void verifyPUKComplete(bool success, SIMError error);

    //! Emitted when pinAttemptsLeft() call completes
    /*!
     * \param attempts number of PIN or PIN2 verification attempts
     * left, -1 if unknown.
     * \param error possible error code, if the operation failed. */
    void pinAttemptsLeftComplete(int attempts, SIMError error);

    //! Emitted when pukAttemptsLeft() call completes
    /*!
     * \param attempts number of PUK or PUK2 verification attempts
     *  left, -1 if unknown.
     * \param error possible error code, if the operation failed. */
    void pukAttemptsLeftComplete(int attempts, SIMError error);

private:
    SIMSecurityPrivate *d;
};

}

}

#endif
