/*
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 */

#ifndef SIMLOCK_H
#define SIMLOCK_H

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QVariant>

#include "libcellular-qt_global.h"

namespace Cellular {

class SIMLockPrivate;

//! SIM lock.
/*!
 * Provides SIMLock status and unlocking.
 *
 * Note that all methods are asynchronous and have a corresponding
 * completion/result signal. */
class CELLULAR_QT_EXPORT SIMLock: public QObject
{
    Q_OBJECT

public:

    //! Possible SIM errors
    enum SIMLockError
    {
	NoError, /*!< Returned when no error happened and operation
		  * completed successfully */
	BusCommunicationError, /*!< Returned when a D-Bus
				* communication error with the
				* Cellular Services daemon occured */
	InvalidParameter, /*!< Invalid parameter values */
	ModemNotReady, /*!< Cellular modem not ready */
	WrongPassword, /*!< Incorrect unlock code */
	AlreadyOpened, /*!< SIMlock already opened */
	WrongLevel, /*!< Requested level not found */
	NotAllowed, /*!< Action not allowed. E.g. the lock does not exist */
	TimerNotElapsed, /*!< Unlocking timer not elapsed (too fast retry) */
	HandlingBlocked, /*!< SIMLock handling blocked (perhaps too many unlock attempts */
	Unknown /*!< General SIM lock unlocking failure */
    };

    //! SIM lock status values
    enum SIMLockStatus
    {
	StatusUnknown, /*!< status not available */
	StatusOk, /*!< SIMLock accepts current (U)SIM card */
	StatusRestricted, /*!< SIMLock rejects the (U)SIM card */ 
	StatusRestrictionPending, /*!< Not all autolocks could clos on
				   * the current (U)SIM card. SIMLock
				   * rejects the card */ 
	StatusRestrictionOn /*!< SIMLock autolocked to current (U)SIM card */ 
    };

    //! Level of SIM lock to be opened
    enum SIMLockLevel
    {
	LevelMccMnc=1,
	LevelGid1,
	LevelGid2,
	LevelImsi,
	LevelMccMncGid1,
	LevelMccMncGid1Gid2,
	LevelGlobal
    };

public:
    SIMLock(QObject *parent=0);
    ~SIMLock();

    //! Check validity of class instance
    bool isValid() const;

public Q_SLOTS: // METHODS
    //! Request current SIMLock status
    void simLockStatus();

    //! unlocks SIM lock.
    void simLockUnlock(SIMLock::SIMLockLevel level, const QString &unlockCode);

Q_SIGNALS: // SIGNALS
    //! Emitted when simLockStatus() call completes
    void simLockStatusComplete(SIMLock::SIMLockStatus status, SIMLock::SIMLockError error);

    //! Emitted when unlock() call completes
    void simLockUnlockComplete(SIMLock::SIMLockError error);

private:
    SIMLockPrivate *d;
};

}

#endif
