/*
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 */

#ifndef SIMPHONEBOOK_H
#define SIMPHONEBOOK_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

#include "libcellular-qt_global.h"

namespace Cellular {

class SIMPhonebookPrivate;

//! SIM phonebook.
/*!
 * This class provides access to SIM phonebooks. It allows getting
 * information about a specific SIM phonebook, and reading, writing or
 * deleting phonebook entries.
 *
 * Note that all methods are asynchronous and have a corresponding
 * completion/result signal.
 */
class CELLULAR_QT_EXPORT SIMPhonebook: public QObject
{
    Q_OBJECT

public:
    //! Possible SIM phonebook errors
    enum SIMPhonebookError
    {
        NoError = 0, /*!< Returned when no error happened and operation
       				     completed successfully */
        BusCommunicationError, /*!< Returned when a D-Bus communication
					     error with the Cellular Services daemon occured */
        DataNotAvailable, /*!< Returned when there is no phonebook entry at
					     the specified index */
        InvalidLocation,
        OperationFailed,
        Parameter,
        Phonet,
        Service,
        Timeout,
        Unknown,
        PhonebookTypeNotSupported,
        InvalidParameter,
        NoSIMService
    };

    //! Supported phonebook types.
    enum PhonebookType
    {
	ADN,
	FDN,
	SDN,
	VMBX,
	MBDN,
	EN,
	MSISDN
    };

    //! Possible FDN phonebook status
    enum FDNStatus
    {
	FDNPhonebookEnabled,
	FDNPhonebookDisabled
    };

    SIMPhonebook(QObject *parent=0);

    ~SIMPhonebook();

    //! Check reliability of readings
    bool isValid() const;

public Q_SLOTS: // METHODS
    //! Creates a phonebook entry. 
    /*!
     * Supported phonebook types are ADN, FDN, VMBX, MBDN. secondName,
     * additionalNumber and email are written only to ADN phonebooks
     * if the SIM card supports that.
     *
     * To write to ADN, VMBX or MBDN phonebooks, PIN code must be
     * verified first.  To write to FDN phonebook, PIN2 code must be
     * verified first. See SIMSecurity class for more details.
     */
    //! @credential %Cellular resource token required
    void createEntry(PhonebookType phonebook, const QString &name,
		     const QString &number, const QString &secondName,
		     const QString &additionalNumber, const QString &email);

    //! Deletes a phonebook entry. 
    /*!
     * Supported phonebook types and PIN verification rules are same
     * as in createEntry() and updateEntry() methods.
     *
     * \param index specifies an index of an entry to be
     * deleted. Possible MBDN indexes are same as in ReadExact()
     * method. After deletion, the rest of the indexes do not
     * change. For example if a phonebook has entries with indexes 0
     * to 3, and entry with an index 1 is deleted, the remaining
     * entries have indexes 0, 2 and 3.
    */
    //! @credential %Cellular resource token required
    void deleteEntry(PhonebookType phonebook, int index);

    //! Returns FDN status via fdnStatusComplete() signal: Enabled or Disabled.
    /*!
     * When FDN phonebook is enabled, phone calls are allowed only to
     * numbers stored in the FDN phonebook.
     */
    void getFDNStatus();

    //! Returns information about a specific SIM phonebook.
    /*!
     * See infoComplete() for the description of what is returned.
     */
    //! @credential %Cellular resource token required
    void info(PhonebookType phonebook);

    //! Read a phonebook entry specified by the index. 
    /*!
     * \param index specifies an index of the wanted entry.
     *
     * For MBDN (Mailbox dialling numbers) phonebook possible indexes are:
     *	- 0: Voicemail
     *	- 1: Fax
     *	- 2: Electronic mail
     *	- 3: Other
     *	- 4: Video
     *
     *	For EN (emergency numbers) possible indexes are 0 to 4.
     */
    //! @credential %Cellular resource token required
    void readExact(PhonebookType phonebook, int index);

    //! Read the first entry in the phonebook
    //! @credential %Cellular resource token required
    void readFirst(PhonebookType phonebook);

    //! Read the entry in the phonebook that follows one specified by the index
    /*!
     * To read all entries, in each iteration except the first one use
     * readNext() with the returned index value from the previous
     * iteration. Stop when
     * Cellular::SIMPhonebook::DataNotAvailable error is
     * returned.
     */
    //! @credential %Cellular resource token required
    void readNext(PhonebookType phonebook, int index);

    //! Enables or disables FDN phonebook
    /*! When FDN phonebook is enabled, phone calls are allowed only to numbers
      stored in the FDN phonebook. Before changing FDN phonebook status, PIN2 code
      must be verified. See SIMSecurity class for more details.
    */
    //! @credential %Cellular resource token required
    void setFDNStatus(FDNStatus status);

    //! Rewrites a phonebook entry.
    /*!
     * \sa createEntry() for PIN code verification rules.
     * \param index An index of the entry to rewrite. For MBDN phonebook
     * possible indexes are same as in readExact() method.
    */
    //! @credential %Cellular resource token required
    void updateEntry(PhonebookType phonebook, int index, const QString &name,
		     const QString &number, const QString &secondName,
		     const QString &additionalNumber, const QString &email);

Q_SIGNALS: // SIGNALS
    //! Emitted when FDN phonebook's status is changed to disabled.
    void fdnDisabled();

    //! Emitted when FDN phonebook's status is changed to enabled.
    void fdnEnabled();

    //! Emitted when createEntry() call has completed.
    /*! \param index An index of the entry that was created. */
    void createEntryComplete(int index, SIMPhonebook::SIMPhonebookError error);

    //! Emitted when deleteEntry() call has completed.
    void deleteEntryComplete(SIMPhonebook::SIMPhonebookError error);

    //! Emitted when getFDNStatus() call has completed.
    /*! \param status FDN status: enabled or disabled.*/
    void fdnStatusComplete(SIMPhonebook::FDNStatus status,
			   SIMPhonebook::SIMPhonebookError error);

    //! Emitted when info() call has completed.
    /*!
     * \param maxEntries maximum number of entries in the phonebook
     * \param maxName maximum length of name field,
     * \param maxNumber maximum length of phone number field,
     * \param maxSNEEntries maximum number of SNE (Second Name) entries in ADN
     * 	phonebook,
     * \param maxSNELength maximum length of SNE entry,
     * \param maxSNREntries maximum number of ANR (Additional Number) entries in
     *  ADN phonebook,
     * \param maxSNRLength maximum length of ANR entry,
     * \param maxEmailEntries maximum number of email address entries in ADN
     *  phonebook,
     * \param maxEmailLength maximum length of email address entry
     */
    void infoComplete(uint maxEntries, uint maxName, uint maxNumber,
		      uint maxSNEEntries, uint maxSNELength, uint maxSNREntries,
		      uint maxSNRLength, uint maxEmailEntries, uint maxEmailLength,
		      SIMPhonebook::SIMPhonebookError error);

    //! Emitted when readExact() call has completed.
    /*
     * \param index index of the returned entry
     *
     *	Second name, additional number and email are returned only for
     *	ADN phonebooks.
     */
    void readExactComplete(int index, QString name, QString number,
			   QString secondName, QString additionalNumber,
			   QString email, SIMPhonebook::SIMPhonebookError error);

    //! Emitted when readFirst() call has completed.
    /*
     * \param index index of the returned entry
     *
     *	Second name, additional number and email are returned only for
     *	ADN phonebooks.
     */
    void readFirstComplete(int index, QString name, QString number,
			   QString secondName, QString additionalNumber,
			   QString email, SIMPhonebook::SIMPhonebookError error);

    //! Emitted when readNext() call has completed.
    /*
     * \param index index of the returned entry
     *
     * Second name, additional number and email are returned only for
     * ADN phonebooks.
     */
    void readNextComplete(int index, QString name, QString number, QString secondName,
			  QString additionalNumber, QString email,
			  SIMPhonebook::SIMPhonebookError error);

    //! Emitted when setFDNStatus() call has completed.
    void setFDNStatusComplete(SIMPhonebook::SIMPhonebookError error);

    //! Emitted when updateEntry() call has completed.
    /*! \param index An index of the entry that was created. */
    void updateEntryComplete(int index, SIMPhonebook::SIMPhonebookError error);

private:
    SIMPhonebookPrivate *d;
};

}

#endif
