/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef SMSMESSAGE_H
#define SMSMESSAGE_H

#include <QtCore/QObject>

#include "libcellular-qt_global.h"

class QString;

namespace Cellular
{

class SMSMessagePrivate;

//! Is used to calculate the on-the-wire size of a text note.  
/*
 * This class is intended for use in the user interface
 * to display the on-the-wire size of a message while it is being
 * composed.
 *
 * The calculation can be controlled by specifying the conversion
 * tables to use. Either the default conversion table can be used,
 * or one of the language variants.
 */
class CELLULAR_QT_EXPORT SMSMessage
{
public:
    //! Enumeration of the possible conversion table variants.
    enum Language 
    { 
        DefaultLanguage = 0, //!< Default conversion table
        Turkish,    //!< Conversion table for the Turkish national variant.
        Spanish,    //!< Conversion table for the Spanish national variant.
        Portuguese, //!< Conversion table for the Portuguese national variant.
	Reduced,    //!< Using the default conversion table with transliterating.
    };

    //! Enumeration of return values to size queries
    enum SMSError
    {
	NoError = 0,    /*!< \a SMSError is not valid, i.e., the call
			 * succeeded. */
	Uninitialized,  /*!< The \a SMSMessage object has not been
			 * properly initialized; therefore its size is
			 * undefined. */
	MessageTooLong, /*!< The text note is too long to fit in a
			 * single (multi-part) SMS message. The
			 * maximum number of segments in a multi-part
			 * SMS is 255, and this error indicates a size
			 * beyond that limit. */
	UnknownUnicode, /*!< There are characters that are
			 * unrepresentable in the SMS charsets. This
			 * happens with unicode codepoints not within
			 * the Basic Multilingual Plane (BMP). */
	UnknownError    /*!< Size could not be calculated due to an
			 * unknown error. */
    };

    SMSMessage();

    //! Constructs an SMSMessage with a specific conversion language.
    /*! 
     * \param lang conversion table to use
     */
    SMSMessage(Language lang);

    //! Constructs an SMSMessage with a specific conversion language
    /*!
     * \param lang conversion table to use
     * \param text  message text
     */
    SMSMessage(Language lang, const QString &text);

    //! Constructs an SMSMessage with message text
    /*!
     * \param text message text
     */
    SMSMessage(const QString &text);

    //! Destructor
    ~SMSMessage();

    //! Sets the contents for this SMSMessage
    /*!
     * This function sets the contents of this SMSMessage to a single
     * plain text body containing \a text.
     *
     * \param test message text
     */
    void setText(const QString &text);


    //! Gets the contents of this message
    /*!
     * This function returns the textual contents of the this SMSMessage.
     *
     * \return Text body of the message
     */
    QString text() const;

    //! Calculates the size of this message as it will appear on-the-wire.
    /*!
     * This function calculates the on-the-wire size of this
     * SMSMessage, setting \a numSegments to the number of required
     * segments if the message needs to be concatenated, and \a
     * spaceInLast to the number of characters available in the last
     * segment.
     *
     * \param numSegments number of segments required
     * \param spaceInLast available space in the last segment
     * \param totalSpace total space available with the currently applicable alphabet
     * \return Returns NoError if calculation successful, and MessageTooLong
     *         if the text cannot fit the maximum size of an SMS.
     */
    SMSError getSize(uint &numSegments, uint &spaceInLast, uint &totalSpace) const;

    //! Deprecated
    SMSError getSize(uint &numSegments, uint &spaceInLast) const;

private:
    SMSMessagePrivate *d;
};

}

#endif  /* !SMSMESSAGE_H */
