/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2010 Nokia Corporation. All rights reserved.
 */

#ifndef SUPPLEMENTARYSERVICES_H
#define SUPPLEMENTARYSERVICES_H

#include <QtCore/QObject>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QVariant>

#include "libcellular-qt_global.h"

namespace Cellular
{

class SupplementaryServicesPrivate;

//! Supplementary Services
/*!
 * Provides access to Supplementary Services. This
 * class is used for registering, erasing, activating, deactivating and
 * interrogating various supplementary services.
 *
 * This service permits a called mobile subscriber to have the network
 * send all incoming calls, or just in some specific case (e.g. when
 * busy with another call), addressed to the called mobile
 * subscriber's directory number to another directory number. The
 * ability of the served mobile subscriber to originate calls is
 * unaffected.
 *
 * This class also provides access to Unstructured Supplementary
 * Service Data (USSD). USSD allows sending unstructured commands
 * towards the network, receiving responses to such commands, as well
 * as receiving unsolicited USSD messages from the network.
 */
class CELLULAR_QT_EXPORT SupplementaryServices : public QObject
{

Q_OBJECT

public:
    SupplementaryServices(QObject *parent=0);
    ~SupplementaryServices();

    //! Supplementary service operations
    /*!
     * These operations are defined in 3GPP TS 22.030
     */
    enum Operation
    {
	UnknownOperation = -1,
	Register,
	Activate,
	Deactivate,
	Erase,
	Interrogate
    };

    //! Supplementary Service Codes valid for divert service
    /*!
     * These codes are defined in 3GPP TS 22.030 (aka MMI codes)
     */
    enum DivertServiceCode
    {
	UnknownDivertService = -1,
	AllCallForwardings = 002,
	CallForwardingsConditional = 004,
	CallForwardingUnconditional = 21, //!< CFU
	CallForwardingNoReply = 61, //!< CFNRY
	CallForwardingNotReachable = 62, //!< CFNRC
	CallForwardingBusy = 67, //!< CFB
    };

    //! Supplementary Service Codes valid for barring service
    /*!
     * These codes are defined in 3GPP TS 22.030 (aka MMI codes)
     */
    enum BarringServiceCode
    {
	UnknownBarringService = -1,
	AllBarrings = 330,
	AllBarringsOfOutgoingCalls = 333,
	BarringOfAllOutgoingCalls = 33, //!< BAOC
	BarringOfOutgoingInternationalCalls = 331, //!< BOIC
	BarringOfOutgoingInternationalCallsExceptToHomeCountry = 332, //!< BOIC_exHC
	AllBarringsOfIncomingCalls = 353,
	BarringOfAllIncomingCalls = 35, //!< BAIC
	BarringOfIncomingCallsWhenRoaming = 351, //!< BIC_Roam
    };

    //! Supplementary Service Codes for other services
    /*!
     * These codes are defined in 3GPP TS 22.030 (aka MMI codes)
     */
    enum OtherServiceCode
    {
	UnknownSupplementaryService = -1, //!< Unknown Supplementary Service Code.
	ServicePasswordChange = 03, //!< Service password change
	CallingLineIdentificationPresentation = 30, //!< CLIP supplementary service
	CallingNamePresentation = 300, //!< CNAP supplementary service
	CallingLineIdentificationRestriction = 31, //!< CLIR supplementary service
	CallWaiting = 43, //!< WAIT
	ConnectedLineIdentificationPresentation = 76, //!< COLP
	ConnectedLineIdentificationRestriction = 77, //!< COLR
    };

    //! Supplementary Services Basic Service Codes
    /*!
     * These codes represent a subset of all available basic service
     * codes for the most typical of services.
     */
    enum BasicService
    {
	UnknownBasicService = -1,
	AllBasicServices = 0, //!< All teleservices as well as data (bearer) services.
	AllTeleServices = 10, //!< Just the teleservices
	Telephony = 11, //!< Telephony service
	AllDataTeleServices = 12,
	Faximile = 13, //!< Fax service
	ShortMessagingService = 16, //!< SMS service
	VoiceGroupCallService = 17,
	VoiceBroadcastService = 18,
	AllTeleservicesExceptSMS = 19,
	AllBearerServices = 20,
	AllAsyncServices = 21,
	AllSyncServices = 22,
	AllDataCircuitSync = 24,
	AllDataCircuitAsync = 25,
	AllDedicatedPacketAccess = 26,
	AllDedicatedPADAccess = 27
    };
    typedef QList<BasicService> BasicServiceList;

    //! Divert service object
    /*!
     * This object is returned as a response to a check for call
     * forwarding supplementary service status.
     */
    struct DivertService {
        BasicService basicService;    //!< Basic service code.
        bool active;             //!< Is the service active.
        QString telephoneNumber; //!< Target telephone number.
        uint forwardDelay;        //!< Forwarding delay setting, in seconds.
    };
    typedef QList<DivertService> DivertServiceList;


    //! Possible CLIR service statuses
    enum ClirStatus 
    {
        ClirUnknown = -1,
	ClirPermanent, //!< CLIR is permanently provisioned
	ClirOn, //!< CLIR is on by default, but can be deactivated on per call basis
	ClirOff, //!< CLIR is off by default, but can be activated on per call basis
    };

    //! Error codes
    /*!
     * These error codes are returned as part of the completed
     * signals. Success means no error, naturally.
     */
    enum Error
    {
	UnknownError = -1, //!< Unknown error code
	Success = 0, //!< Operation was successful
	Failure, //!< Operation failed
	NotProvisioned, //!< Supplementary service not provisioned
	WrongPassword, //!< Incorrect password
	InvalidTelephoneNumber, //!< Invalid telephone number when trying to register a divert
	OverridenError, //!< Another active service prevented activation of requested service
	Incompatibility, //!< Requested operation is not compatible with a status of another supplementary service or a basic service
	NotAvailable, //!< Supplementary service is not available in the current location
	TooManyPasswordAttempts, //!< Maximum number of wrong password attempts is exceeded
	SubscriptionViolation, //!< Subscription restricts this operation
	FixedDialNumbers, //!< Fixed Dialing Numbers is activated and restricts this operation
	Busy, //!< Server is busy
        Suspended //!< Supplementary services not available
    };
    
    //! USSD operations
    /*!
     * These operations are returned by ussdError() signal. That makes
     * it possible to know which operation has caused the error.
     */
    enum UssdOp
    {
	UssdCommand,
	UssdReply,
	UssdNotifyAck,
	UssdRelease
    };    

    //! USSD Error codes
    /*!
     * These error codes are returned by ussdError() signal.
     */
    enum UssdError
    {
	UnknownUssdError, //!< Unknown error code
	UssdFailure, //!< Operation failed
	UssdInvalidParameters,
	UssdBusy,
	UssdOffline,
	UssdNetwork,
	UssdNetworkBusy,
	UssdNoModemService,
	UssdFixedDialNumbers,
	UssdMissingRequest,
	UssdMissingNotify,
	UssdBadData,
	UssdReleased,
	UssdSuspended,
	UssdSystemFailure,
	UssdCallBarred,
	UssdDataMissing,
	UssdUnexpectedDataValue,
	UssdUnknownAlphabet
    };

    //! Check availability of the API
    bool isValid() const;

public Q_SLOTS:
    //! Network is busy with an ongoing service request
    /*!
     * Check whether the network is busy servicing an ongoing service
     * request, or if supplementary service requests are currently
     * likely to succeed.
     *
     * \returns True if the network is currently busy, False
     * otherwise.
     */
    bool isBusy();

    //! Registers and activates call forwarding.
    /*!
     * \param supplementaryService Condition to forward call.
     * \param basicService What kind of calls are being forwarded.
     * \param telephoneNumber Destination phone number where calls are
     * to be forwarded.
     * \param forwardDelay Delay in seconds, after which call gets
     *  forwarded. Nonzero value is allowed only for
     *  AllCallForwardings, CallForwardingsConditional, and
     *  CallForwardingNoReply supplementary service codes. Valid
     *  values are 5, 10, 15, 20, 25 and 30 seconds.
     *
     * The service responds by emitting registerDivertCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void registerDivert(DivertServiceCode supplementaryService,
			BasicService basicService,
			const QString &telephoneNumber,
			uint forwardDelay = 0);

    //! Erase and deactivate call forwarding
    /*!
     * \param supplementaryService Type of call forwarding service.
     * \param basicService The type of calls being forwarded.
     *
     * The service responds by emitting eraseDivertCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void eraseDivert(DivertServiceCode supplementaryService, BasicService basicService);

    //! Activate call forwarding
    /*!
     * \param supplementaryService Type of call forwarding service.
     * \param basicService The type of calls being forwarded.
     *
     * The service responds by emitting activateDivertCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void activateDivert(DivertServiceCode supplementaryService, BasicService basicService);

    //! Deactivate call forwarding
    /*!
     * \param supplementaryService Type of call forwarding service.
     * \param basicService The type of calls being forwarded.
     *
     * The service responds by emitting deactivateDivertCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void deactivateDivert(DivertServiceCode supplementaryService, BasicService basicService);

    //! Check status of call forwarding
    /*!
     * \param supplementaryService Type of call forwarding service.
     * \param basicService The type of calls being forwarded.
     *
     * The service responds by emitting checkDivertCompleted()
     * signal.
     */
    void checkDivert(DivertServiceCode supplementaryService, BasicService basicService);

    //! Change a supplementary service password
    /*!
     * \param supplementaryService Type of supplementary service.
     * \param oldPassword Old password.
     * \param newPassword New password to replace \a oldPassword.
     *
     * The service responds by emitting changePasswordCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void changeBarringPassword(BarringServiceCode supplementaryService,
			const QString &oldPassword, const QString &newPassword);

    //! Activate call barring
    /*!
     * \param supplementaryService Type of call barring service.
     * \param basicService The type of calls being barred.
     *
     * The service responds by emitting activateBarringCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void activateBarring(BarringServiceCode supplementaryService,
			 BasicService basicService,
			 const QString &password);

    //! Deactivate call barring
    /*!
     * \param supplementaryService Type of call barring service.
     * \param basicService The type of calls being barred.
     *
     * The service responds by emitting deactivateBarringCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void deactivateBarring(BarringServiceCode supplementaryService,
			   BasicService basicService,
			   const QString &password);

    //! Check call barring
    /*!
     * \param supplementaryService Type of call barring service.
     *
     * The service responds by emitting checkBarringCompleted()
     * signal.
     */
    void checkBarring(BarringServiceCode supplementaryService);

    //! Activate call waiting
    /*!
     * \param basicService Target calls for call waiting.
     *
     * The service responds by emitting activateWaitingCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void activateWaiting(BasicService basicService);

    //! Deactivate call waiting
    /*!
     * \param basicService Target calls for call waiting.
     *
     * The service responds by emitting deactivateWaitingCompleted()
     * signal.
     */
    //! @credential %Cellular resource token required
    void deactivateWaiting(BasicService basicService);

    //! Check call waiting status
    /*!
     * \param basicService Target calls for call waiting.
     *
     * The service responds by emitting checkWaitingCompleted()
     * signal.
     */
    void checkWaiting(BasicService basicService);

    //! Check the CLIR service status
    /*!
     * This method is for checking the Calling Line Identification
     * Restriction (CLIR) supplementary service setting.
     *
     * The service responds by emitting checkClirCompleted()
     * signal.
     */
    void checkClir();

    //! Check the CLIP service status
    /*!
     * This method is for checking the Calling Line Identification
     * Presentation (CLIP) supplementary service setting.
     *
     * The service responds by emitting checkClipCompleted()
     * signal.
     */
    void checkClip();

    //! Check the COLP service status
    /*!
     * This method is for checking the Connected Line Identification
     * Presentation (COLP) supplementary service setting.
     *
     * The service responds by emitting checkColpCompleted()
     * signal.
     */
    void checkColp();

    //! Check the COLR service status
    /*!
     * This method is for checking the Connected Line Identification
     * Restriction (COLR) supplementary service setting.
     *
     * The service responds by emitting checkColrCompleted()
     * signal.
     */
    void checkColr();

    //! Check the CNAP service status
    /*!
     * This method is for checking the Connected Name Presentation
     * (CNAP) supplementary service setting.
     *
     * The service responds by emitting checkCnapCompleted()
     * signal.
     */
    void checkCnap();

    //! Check whether a string contains a service code
    /*!
     * Check whether a string is formatted as a valid, well-known
     * supplementary service operation or a USSD command.
     *
     * \param command String containing a possible operation.
     * \return True if the string contains a supplementary service
     * operation or USSD command, False otherwise.
     */
    bool isServiceCode(const QString &command);

    //! Send a supplementary service request
    /*!
     * Send a supplementary service request via the conventional way defined in the
     * MMI document (TS 22.030). The request is submitted as a string, including special codes 
     * starting with * or # characters, followed by digits or more * # characters and ending with #.
     * These special codes are known as 'service procedures' and they are used to
     * launch different supplementary service operations.
     *
     * \param command String containing a possible operation.
     * \return True if the string contains a valid supplementary service
     * operation that was initiated, False otherwise.
     */
    //! @credential %Cellular resource token required
    bool sendSSRequest(const QString &command);

    //! Send a USSD command
    /*!
     * Send a USSD command. 
     *
     * The service responds by emitting a commandCompleted() signal
     * when the command completed (or ussdError() if an error has occured). Additionally, a
     * userActionRequested() signal may be emitted, if the network
     * requested user action after receiving a USSD request. Use sendReply()
     * to return a reply to that.
     */
    //! @credential %Cellular resource token required
    void sendCommand(const QString &command);

    //! Return a reply to a request by network 
    /*!
     * Return a reply to a request sent by network via userActionRequested()
     * signal.
     *
     * The service responds by emitting a replyCompleted() signal
     * when the command completed(or ussdError() if an error has occured). Additionally, another
     * userActionRequested() signal may be emitted, if the network
     * requested further user action after receiving this reply. 
     * Use another sendReply() to answer that.
     */
    //! @credential %Cellular resource token required
    void sendReply(const QString &command);

    //! Send acknowledgment of an USSD message received from the network
    /*!
     * This command sends an acknowledgement of an USSD message received from the network
     * via notifyReceived() signal. Further commands can be sent or received after notifyAckCompleted()
     * signal has arrived.
     */
    //! @credential %Cellular resource token required
    void sendNotifyAck();
     
    //! End an USSD session
    /*!
     * This commands ends an USSD session. The service responds by emitting
     * ussdReleased() (or ussdError() if an error has occured).
     */
    //! @credential %Cellular resource token required
    void ussdRelease();

Q_SIGNALS:
    //! Network status changed
    void statusChanged(bool busy);

    //! Response to a registerDivert() when the request has completed.
    void registerDivertCompleted(SupplementaryServices::Error error);

    //! Response to an eraseDivert() when the request has completed.
    void eraseDivertCompleted(SupplementaryServices::Error error);

    //! Response to an activateDivert() when the request has completed.
    void activateDivertCompleted(SupplementaryServices::Error error);

    //! Response to a deactivateDivert() when the request has completed.
    void deactivateDivertCompleted(SupplementaryServices::Error error);

    //! Response to a checkDivert() when the request has completed.
    void checkDivertCompleted(SupplementaryServices::DivertServiceList diversions, SupplementaryServices::Error error);

    //! Response to a changeBarringPassword() when the request has completed.
    void changeBarringPasswordCompleted(SupplementaryServices::Error error);

    //! Response to an activateBarring() when the request has completed.
    void activateBarringCompleted(SupplementaryServices::Error error);

    //! Response to a deactivateBarring() when the request has completed.
    void deactivateBarringCompleted(SupplementaryServices::Error error);

    //! Response to a checkBarring() when the request has completed.
    void checkBarringCompleted(SupplementaryServices::BasicServiceList baddings, SupplementaryServices::Error error);

    //! Response to an activateWaiting() when the request has completed.
    void activateWaitingCompleted(SupplementaryServices::Error error);

    //! Response to a deactivateWaiting() when the request has completed.
    void deactivateWaitingCompleted(SupplementaryServices::Error error);

    //! Response to a checkWaiting() when the request has completed.
    void checkWaitingCompleted(SupplementaryServices::BasicServiceList waitings, SupplementaryServices::Error error);

    //! Response to a checkClir() when the request has completed.
    void checkClirCompleted(SupplementaryServices::ClirStatus status, SupplementaryServices::Error error);

    //! Response to a checkClip() when the request has completed.
    void checkClipCompleted(bool result, SupplementaryServices::Error error);

    //! Response to a checkColp() when the request has completed.
    void checkColpCompleted(bool result, SupplementaryServices::Error error);

    //! Response to a checkColr() when the request has completed.
    void checkColrCompleted(bool result, SupplementaryServices::Error error);

    //! Response to a checkCnap() when the request has completed.
    void checkCnapCompleted(bool result, SupplementaryServices::Error error);

    //! USSD command completed.
    void commandCompleted(const QString &result);

    //! USSD reply completed.
    void replyCompleted(const QString &result);

    //! Notify acknowldgement has completed.
    void notifyAckCompleted();

    //! Network is expecting user action, and the USSD session is left open.
    void userActionRequested(const QString &result);

    //! Network has sent a Mobile Terminated (MT) USSD message.
    void notifyReceived(const QString &message);

    //! USSD session has been closed. Contains optional final response from the network.
    void ussdReleased(const QString &message);

    //! USSD error has occured
    void ussdError(SupplementaryServices::UssdOp operation, SupplementaryServices::UssdError error, QString message);

private:
    SupplementaryServicesPrivate *d;
};

}

#endif  /* !SUPPLEMENTARYSERVICES_H */
