/**
 * This file is part of CellularQt
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 */

#ifndef SYSTEMCONTROL_H
#define SYSTEMCONTROL_H

#include <QtCore/QObject>
#include <QtCore/QFlags>

#include "libcellular-qt_global.h"

namespace Cellular
{

class SystemControlPrivate;

//! Cellular system control.
/*!
 * This class provides basic information of the cellular system status
 * and modem health. It also allows controlling the radio interface so
 * that it's possible to take the device into so called flight-mode by
 * disabling the cellular radio.
 */
class CELLULAR_QT_EXPORT SystemControl : public QObject
{

    Q_OBJECT
    Q_PROPERTY(bool power READ power WRITE setPower)
    Q_PROPERTY(Status status READ status)
    Q_PROPERTY(Activity activity READ activity)
    Q_PROPERTY(bool valid READ isValid)
    Q_ENUMS(Status Activity)

public:
    SystemControl(QObject *parent=0);
    ~SystemControl();

    enum Status {
	UnknownStatus = -1, //!< Current cellular service status is unknown.
	FullService,        //!< Cellular system is in full service.
	LimitedService,     //!< Cellular system is in limited service, i.e., 
                            //   only emergency services are available.
	NoService,          //!< Cellular services unavailable because of no network coverage.
	ModemFailure,       //!< Modem self-tests failed, take the device into service.
    };

    enum Activity {
	UnknownActivity = -1, //!< Cellular system's current activity is unknown
	Idle,                 //!< Cellular system is idle
	Signaling,            //!< Cellular system is signaling
	Call,                 //!< Cellular system has an ongoing call
	Data                  //!< Cellualr system has an active data connection
    };

    //! Get the current cellular system's radio status.
    bool power() const;
    //! Get the current system status.
    Status status() const;
    //! Get the current system activity
    Activity activity() const;
    //! Power the cellular radio on or off.
    //! @credential %Cellular resource token required
    void setPower(bool value);
    //! Check validity of class instance.
    bool isValid() const;

signals:
    //! Power state of the cellular radio changed.
    void powerChanged(bool value);
    //! Cellular service status changed.
    void statusChanged(int status);
    //! Cellular system activity changed
    void activityChanged(int activity);

private:
    SystemControlPrivate *d;
};

}
#endif  /* !SYSTEMCONTROL_H */
